package hpdos.lib;

import hpdos.message.MessageConstants;

import java.util.HashMap;

public class MemoryStorageService implements StorageService {
    private final HashMap<String, StorageModel> memoryKVStore;
    public MemoryStorageService() {
        this.memoryKVStore = new HashMap<>();
    }
    @Override
    public StoredModel create(String key, StorageModel value) {
        try {
            if (memoryKVStore.containsKey(key))
                return new StoredModel(null, MessageConstants.STATUS_KEY_EXISTS);
            memoryKVStore.put(key, value);
            return new StoredModel(value, MessageConstants.STATUS_OK);
        } catch (Exception e) {
            return null;
        }
    }

    @Override
    public StoredModel readByKey(String key) {
        if (!memoryKVStore.containsKey(key))
            return new StoredModel(null, MessageConstants.STATUS_KEY_NOT_FOUND);
        return new StoredModel(memoryKVStore.get(key), MessageConstants.STATUS_OK);
    }

    @Override
    public StoredModel update(String key, StorageModel value) {
        if (!memoryKVStore.containsKey(key))
            return new StoredModel(null, MessageConstants.STATUS_KEY_NOT_FOUND);
        StorageModel previousValue = memoryKVStore.get(key);
        if (previousValue.getVersion() != value.getVersion())
            return new StoredModel(previousValue, MessageConstants.STATUS_UPDATE_VERSION_MISMATCH);
        value.setVersion((previousValue.getVersion() + 1) % Integer.MAX_VALUE); // version wraps around
        memoryKVStore.put(key, value);
        return new StoredModel(value, MessageConstants.STATUS_OK);
    }

    @Override
    public StoredModel delete(String key, int version) {
        if (!memoryKVStore.containsKey(key))
            return new StoredModel(null, MessageConstants.STATUS_KEY_NOT_FOUND);
        StorageModel previousValue = memoryKVStore.get(key);
        if (previousValue.getVersion() != version)
            return new StoredModel(previousValue, MessageConstants.STATUS_UPDATE_VERSION_MISMATCH);
        return new StoredModel(memoryKVStore.remove(key), MessageConstants.STATUS_OK);
    }
}
